#include <gtk/gtk.h>
#include <gst/gst.h>

static GstElement *pipeline;
static gboolean recording = FALSE;
static gchar *filename;

static void
toggle_recording(GtkButton *button, gpointer user_data) {
    if (!recording) {
        GDateTime *now = g_date_time_new_now_local();
        filename = g_date_time_format(now, "audio_%Y-%m-%d_%H-%M-%S.ogg");
        g_date_time_unref(now);

        pipeline = gst_parse_launch("autoaudiosrc ! audioconvert ! vorbisenc ! oggmux ! filesink name=sink", NULL);
        GstElement *sink = gst_bin_get_by_name(GST_BIN(pipeline), "sink");
        g_object_set(sink, "location", filename, NULL);
        gst_object_unref(sink);

        gst_element_set_state(pipeline, GST_STATE_PLAYING);
        recording = TRUE;
        gtk_button_set_label(button, "Stopp opptak");
    } else {
        gst_element_set_state(pipeline, GST_STATE_NULL);
        gst_object_unref(pipeline);
        recording = FALSE;
        gtk_button_set_label(button, "Start opptak");
        g_print("Opptak lagret: %s\n", filename);
        g_free(filename);
    }
}

static void
activate(GtkApplication *app, gpointer user_data) {
    GtkWidget *window = gtk_application_window_new(app);
    gtk_window_set_title(GTK_WINDOW(window), "GNOME Audio Locator");
    gtk_window_set_default_size(GTK_WINDOW(window), 400, 200);

    GtkWidget *button = gtk_button_new_with_label("Start opptak");
    g_signal_connect(button, "clicked", G_CALLBACK(toggle_recording), NULL);
    gtk_window_set_child(GTK_WINDOW(window), button);

    gtk_window_present(GTK_WINDOW(window));
}

int main(int argc, char **argv) {
    GtkApplication *app;
    int status;

    gst_init(&argc, &argv);
    app = gtk_application_new("com.aamot.AudioLocator", G_APPLICATION_FLAGS_NONE);
    g_signal_connect(app, "activate", G_CALLBACK(activate), NULL);
    status = g_application_run(G_APPLICATION(app), argc, argv);
    g_object_unref(app);

    return status;
}
